@@ -361,27 +361,27 @@ void KateSyntaxDocument::setupModeList (bool force)
     return;
 
   // We'll store the ModeList in katesyntaxhighlightingrc
-  KConfigGroup config(m_config, "General");
+  KConfigGroup generalConfig(m_config, "General");
 
   // figure our if the kate install is too new
-  if (config.readEntry ("Version",0) > config.readEntry ("CachedVersion",0))
+  if (generalConfig.readEntry ("Version",0) > generalConfig.readEntry ("CachedVersion",0))
   {
-    config.writeEntry ("CachedVersion", config.readEntry ("Version",0));
+    generalConfig.writeEntry ("CachedVersion", generalConfig.readEntry ("Version",0));
     force = true;
   }
 
   // Let's get a list of all the xml files for hl
-  QStringList list = KGlobal::dirs()->findAllResources("data","katepart/syntax/*.xml",
+  const QStringList list = KGlobal::dirs()->findAllResources("data","katepart/syntax/*.xml",
                                                        KStandardDirs::NoDuplicates);
 
   // Let's iterate through the list and build the Mode List
-  for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
+  for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it )
   {
     // Each file has a group called:
     QString Group="Cache "+ *it;
 
     // Let's go to this group
-    config.changeGroup(Group);
+    KConfigGroup config(m_config, Group);
 
     // stat the file
     struct stat sbuf;
@@ -400,6 +400,7 @@ void KateSyntaxDocument::setupModeList (bool force)
       mli->extension  = config.readEntry("extension");
       mli->version    = config.readEntry("version");
       mli->priority   = config.readEntry("priority");
+      mli->style      = config.readEntry("style");
       mli->author    = config.readEntry("author");
       mli->license   = config.readEntry("license");
       mli->hidden   =  config.readEntry("hidden", false);
@@ -446,6 +447,7 @@ void KateSyntaxDocument::setupModeList (bool force)
               mli->extension = root.attribute("extensions");
               mli->version   = root.attribute("version");
               mli->priority  = root.attribute("priority");
+              mli->style     = root.attribute("style");
               mli->author    = root.attribute("author");
               mli->license   = root.attribute("license");
 
@@ -455,13 +457,14 @@ void KateSyntaxDocument::setupModeList (bool force)
               mli->identifier = *it;
 
               // Now let's write or overwrite (if force==true) the entry in katesyntax...rc
-              config.changeGroup(Group);
+              config = KConfigGroup(m_config, Group);
               config.writeEntry("name",mli->name);
               config.writeEntry("section",mli->section);
               config.writeEntry("mimetype",mli->mimetype);
               config.writeEntry("extension",mli->extension);
               config.writeEntry("version",mli->version);
               config.writeEntry("priority",mli->priority);
+              config.writeEntry("style",mli->style);
               config.writeEntry("author",mli->author);
               config.writeEntry("license",mli->license);
               config.writeEntry("hidden",mli->hidden);
@@ -496,8 +499,8 @@ void KateSyntaxDocument::setupModeList (bool force)
     }
   }
 
-  // Syncronize with the file katesyntax...rc
-  config.sync();
+  // Synchronize with the file katesyntax...rc
+  generalConfig.sync();
 }
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
