@@ -53,11 +53,19 @@ private Q_SLOTS:
     void testGroupEscape();
     void testRevertAllEntries();
     void testChangeGroup();
+    void testGroupCopyTo();
     void testConfigCopyTo();
+    void testConfigCopyToSync();
+    void testReparent();
     void testKdeglobals();
     void testSubGroup();
     void testAddConfigSources();
+    void testWriteOnSync();
+    void testDirtyOnEqual();
     void testCreateDir();
+    void testSharedConfig();
+    void testOptionOrder();
+    void testLocaleConfig();
 
     // unrelated
     void testKAboutDataOrganizationDomain();
