@@ -27,8 +27,11 @@
 #include "kaction.h"
 #include "kaction_p.h"
 #include "kglobalaccel_p.h"
+#include "klocale.h"
+#include "kmessagebox.h"
 
 #include <QtGui/QApplication>
+#include <QtGui/QShortcutEvent>
 
 #include <kdebug.h>
 
@@ -50,6 +53,42 @@ void KActionPrivate::init(KAction *q_ptr)
   q->setProperty("isShortcutConfigurable", true);
 }
 
+void KActionPrivate::setActiveGlobalShortcutNoEnable(const KShortcut &cut)
+{
+    globalShortcut = cut;
+    emit q->globalShortcutChanged(cut.primary());
+}
+
+
+void KActionPrivate::slotTriggered()
+{
+#ifdef KDE3_SUPPORT
+  emit q->activated();
+#endif
+  emit q->triggered(QApplication::mouseButtons(), QApplication::keyboardModifiers());
+}
+
+
+bool KAction::event(QEvent *event)
+{
+    if (event->type() == QEvent::Shortcut) {
+        QShortcutEvent *se = static_cast<QShortcutEvent*>(event);
+        if(se->isAmbiguous()) {
+            KMessageBox::information(
+                    NULL,  // No widget to be seen around here
+                    i18n( "The key sequence '%1' is ambiguous. Use 'Configure Shortcuts'\n"
+                          "from the 'Settings' menu to solve the ambiguity.\n"
+                          "No action will be triggered.",
+                                se->key().toString()),
+                    i18n("Ambiguous shortcut detected"));
+            return true;
+        }
+    }
+
+    return QAction::event(event);
+}
+
+
 //---------------------------------------------------------------------
 // KAction
 //---------------------------------------------------------------------
@@ -144,14 +183,6 @@ void KAction::setShortcuts(const QList<QKeySequence>& shortcuts, ShortcutTypes t
   setShortcut(KShortcut(shortcuts), type);
 }
 
-void KActionPrivate::slotTriggered()
-{
-#ifdef KDE3_SUPPORT
-  emit q->activated();
-#endif
-  emit q->triggered(QApplication::mouseButtons(), QApplication::keyboardModifiers());
-}
-
 const KShortcut & KAction::globalShortcut(ShortcutTypes type) const
 {
   Q_ASSERT(type);
@@ -184,7 +215,7 @@ void KAction::setGlobalShortcut( const KShortcut & shortcut, ShortcutTypes type,
 
   if (!d->globalShortcutEnabled) {
     changed = true;
-    if (objectName().isEmpty()) {
+    if (objectName().isEmpty() || objectName().startsWith("unnamed-")) {
       kWarning(283) << "Attempt to set global shortcut for action without objectName()."
                        " Read the setGlobalShortcut() documentation.";
       return;
