@@ -67,8 +67,8 @@ public:
 
     virtual void insertedIntoDocument();
     virtual void removedFromDocument();
-    virtual void addId(const QString& id);
-    virtual void removeId(const QString& id);
+    virtual void addId(const DOMString& id);
+    virtual void removeId(const DOMString& id);
 
 
     long length() const;
@@ -88,7 +88,7 @@ public:
 
     virtual void parseAttribute(AttributeImpl *attr);
 
-    void radioClicked( HTMLGenericFormElementImpl *caller );
+    void uncheckOtherRadioButtonsInGroup(HTMLGenericFormElementImpl *caller, bool setDefaultChecked = false);
 
     void registerFormElement(HTMLGenericFormElementImpl *);
     void removeFormElement(HTMLGenericFormElementImpl *);
@@ -155,7 +155,9 @@ public:
     void setDisabled(bool _disabled);
 
     virtual bool isFocusable() const;
-    virtual bool isEnumeratable() const { return false; }
+    virtual bool isEnumerable() const { return false; }
+    
+    virtual bool isDefault() const { return false; }
 
     bool readOnly() const { return m_readOnly; }
     void setReadOnly(bool _readOnly) { m_readOnly = _readOnly; }
@@ -164,6 +166,7 @@ public:
     void setName(const DOMString& name);
 
     virtual bool isGenericFormElement() const { return true; }
+    virtual bool isHiddenInput() const { return false; }
 
     /*
      * override in derived classes to get the encoded name=value pair
@@ -201,7 +204,7 @@ public:
     };
 
     virtual Id id() const;
-    virtual bool isEnumeratable() const { return true; }
+    virtual bool isEnumerable() const { return true; }
 
     DOMString type() const;
     typeEnum buttonType() const { return KDE_CAST_BF_ENUM(typeEnum, m_type); }
@@ -265,12 +268,15 @@ public:
 
     virtual Id id() const;
 
-    virtual bool isEnumeratable() const { return inputType() != IMAGE; }
+    virtual bool isEnumerable() const { return inputType() != IMAGE; }
+    virtual bool isDefault() const { return m_defaultChecked; }
+    virtual bool isHiddenInput() const { return inputType() == HIDDEN; }
 
     bool autoComplete() const { return m_autocomplete; }
 
+    bool defaultChecked() const { return m_defaultChecked; }
     bool checked() const { return m_useDefaultChecked ? m_defaultChecked : m_checked; }
-    void setChecked(bool);
+    void setChecked(bool, bool setDefaultChecked = false);
     bool indeterminate() const { return m_indeterminate; }
     void setIndeterminate(bool);
     long maxLength() const { return m_maxLen; }
@@ -393,7 +399,7 @@ public:
     long selectedIndex() const;
     void setSelectedIndex( long index );
 
-    virtual bool isEnumeratable() const { return true; }
+    virtual bool isEnumerable() const { return true; }
 
     long length() const;
 
@@ -416,6 +422,7 @@ public:
     virtual NodeImpl *insertBefore ( NodeImpl *newChild, NodeImpl *refChild, int &exceptioncode );
     virtual void      replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
     virtual void      removeChild ( NodeImpl *oldChild, int &exceptioncode );
+    virtual void      removeChildren();
     virtual NodeImpl *appendChild ( NodeImpl *newChild, int &exceptioncode );
     virtual NodeImpl *addChild( NodeImpl* newChild );
 
@@ -468,7 +475,7 @@ public:
     void setSelectedIndex( long index );
 
     // ### this is just a rough guess
-    virtual bool isEnumeratable() const { return false; }
+    virtual bool isEnumerable() const { return false; }
 
     virtual void parseAttribute(AttributeImpl *attr);
     virtual bool encoding(const QTextCodec*, khtml::encodingList&, bool);
@@ -507,7 +514,13 @@ public:
     DOMString value() const;
     void setValue(DOMStringImpl* value);
 
-    bool selected() const { return m_selected; }
+    virtual bool isDefault() const { return m_defaultSelected; }    
+
+	// For internal use --- just returns the bit
+    bool selectedBit() const { return m_selected; }
+
+	// DOM use --- may recompute information of a parent <select>
+	bool selected() const;
     void setSelected(bool _selected);
     void setDefaultSelected( bool _defaultSelected );
 
@@ -516,6 +529,7 @@ public:
 protected:
     DOMString m_value;
     bool m_selected;
+    bool m_defaultSelected;
 };
 
 
@@ -544,7 +558,7 @@ public:
 
     WrapMethod wrap() const { return m_wrap; }
 
-    virtual bool isEnumeratable() const { return true; }
+    virtual bool isEnumerable() const { return true; }
 
     DOMString type() const;
 
