@@ -92,7 +92,7 @@ public:
 
     DOM::HTMLGenericFormElementImpl *element() const
     { return static_cast<DOM::HTMLGenericFormElementImpl*>(RenderObject::element()); }
-    
+
     // this is not a virtual function
     void setQWidget(QWidget *w);
 
@@ -102,10 +102,11 @@ protected:
 	Qt::AlignmentFlag textAlignment() const;
 
     virtual void setPadding();
+    KdeUiProxyStyle* getProxyStyle();
 
 //     QPoint m_mousePos;
 //     int m_state;
-    KdeUiProxyStyle *proxyStyle;
+    KdeUiProxyStyle *m_proxyStyle;
 };
 
 // -------------------------------------------------------------------------
@@ -119,7 +120,10 @@ public:
 
     virtual const char *renderName() const { return "RenderButton"; }
 
+    virtual void layout();
     virtual short baselinePosition( bool ) const;
+    virtual void setStyle(RenderStyle *style);
+    virtual bool forceTransparentText() const { return m_hasTextIndentHack; }
 
     // don't even think about making this method virtual!
     DOM::HTMLInputElementImpl* element() const
@@ -127,12 +131,14 @@ public:
 
 protected:
     virtual bool isRenderButton() const { return true; }
+
+    bool m_hasTextIndentHack;
 };
 
 // -------------------------------------------------------------------------
 
-class CheckBoxWidget: public QCheckBox, public KHTMLWidget 
-{ 
+class CheckBoxWidget: public QCheckBox, public KHTMLWidget
+{
 public:
     CheckBoxWidget(QWidget *p): QCheckBox(p) { m_kwp->setIsRedirected(true); }
 };
@@ -159,8 +165,8 @@ private:
 
 // -------------------------------------------------------------------------
 
-class RadioButtonWidget: public QRadioButton, public KHTMLWidget 
-{ 
+class RadioButtonWidget: public QRadioButton, public KHTMLWidget
+{
 public:
     RadioButtonWidget(QWidget* p): QRadioButton(p) { m_kwp->setIsRedirected(true); }
 };
@@ -189,8 +195,8 @@ private:
 
 // -------------------------------------------------------------------------
 
-class PushButtonWidget: public QPushButton, public KHTMLWidget 
-{ 
+class PushButtonWidget: public QPushButton, public KHTMLWidget
+{
 public:
     PushButtonWidget(QWidget* p): QPushButton(p) { m_kwp->setIsRedirected(true); }
 };
@@ -205,6 +211,8 @@ public:
     virtual void calcMinMaxWidth();
     virtual void updateFromElement();
     virtual short baselinePosition( bool ) const;
+protected:
+    virtual bool canHaveBorder() const;
 private:
     QString rawText();
 };
@@ -256,6 +264,7 @@ public:
     virtual void updateFromElement();
     virtual void setStyle(RenderStyle *style);
     virtual short baselinePosition( bool ) const;
+    virtual void handleFocusOut();
 
     void select();
 
@@ -272,8 +281,8 @@ public:
 public Q_SLOTS:
     void slotReturnPressed();
     void slotTextChanged(const QString &string);
+    void slotCompletionBoxActivated(const QString &string);
 protected:
-    virtual void handleFocusOut();
 
 private:
     virtual bool isEditable() const { return true; }
@@ -301,6 +310,7 @@ private Q_SLOTS:
     void clearHistoryActivated();
     void slotCheckSpelling();
     void slotSpellCheckDone( const QString &s );
+    void slotCreateWebShortcut();
     void spellCheckerMisspelling( const QString &text, int pos);
     void spellCheckerCorrected( const QString &, int, const QString &);
     void spellCheckerFinished();
@@ -371,9 +381,9 @@ protected:
     virtual bool isEditable() const { return true; }
     virtual bool canHaveBorder() const { return true; }
     virtual bool acceptsSyntheticEvents() const { return false; }
-    
+
     virtual bool includesPadding() const { return false; }
-    
+
     bool m_clicked;
     bool m_haveFocus;
 };
@@ -413,14 +423,15 @@ public:
 protected:
     virtual bool event(QEvent *);
     virtual bool eventFilter(QObject *dest, QEvent *e);
+    virtual void keyPressEvent(QKeyEvent *e);
     virtual void showPopup();
     virtual void hidePopup();
 };
 
 // -------------------------------------------------------------------------
 
-class ListBoxWidget: public KListWidget, public KHTMLWidget 
-{ 
+class ListBoxWidget: public KListWidget, public KHTMLWidget
+{
 public:
     ListBoxWidget(QWidget* p): KListWidget(p) { m_kwp->setIsRedirected(true); }
 protected:
@@ -481,28 +492,9 @@ public:
 
 protected:
     virtual bool event (QEvent *e );
-    virtual void contextMenuEvent(QContextMenuEvent * e);
+    virtual void keyPressEvent(QKeyEvent *e);
     virtual void scrollContentsBy(int dx, int dy);
 
-private Q_SLOTS:
-    void slotFind();
-    void slotDoFind();
-    void slotFindNext();
-    void slotReplace();
-    void slotDoReplace();
-    void slotReplaceNext();
-    void slotReplaceText(const QString&, int, int, int);
-    void slotFindHighlight(const QString&, int, int);
-private:
-    KFindDialog *m_findDlg;
-    KFind *m_find;
-    KReplaceDialog *m_repDlg;
-    KReplace *m_replace;
-    KAction *m_findAction;
-    KAction *m_findNextAction;
-    KAction *m_replaceAction;
-    int m_findIndex, m_findPara;
-    int m_repIndex, m_repPara;
 };
 
 
