@@ -19,6 +19,8 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#define QT_NO_CAST_FROM_ASCII
+
 #include <config.h>
 
 #include <sys/types.h>
@@ -46,13 +48,11 @@
 #include <QtCore/QFile>
 #include <QtCore/QDate>
 #include <QtCore/QFileInfo>
-#include <QtCore/QTextStream>
 #include <QtCore/QRegExp>
 #include <QtGui/QFont>
-#include <QDir>
 #include <kcomponentdata.h>
 #include <kstandarddirs.h>
-#include <kconfiggroup.h>
+#include <kglobalsettings.h>
 #include <kglobal.h>
 #include <kconfig.h>
 #include <klibloader.h>
@@ -89,6 +89,8 @@
 #endif
 #endif
 
+// #define SKIP_PROCTITLE 1
+
 extern char **environ;
 
 #ifdef Q_WS_X11
@@ -256,7 +258,7 @@ static void child_died(pid_t exit_pid, int exit_status)
          /* Send a message with the return value of the child on the control socket */
          klauncher_header request_header;
          long request_data[2];
-         request_header.cmd = LAUNCHER_DIED;
+         request_header.cmd = LAUNCHER_CHILD_DIED;
          request_header.arg_length = sizeof(long) * 2;
          request_data[0] = exit_pid;
          request_data[1] = exit_status;
@@ -375,7 +377,7 @@ static void init_startup_info( KStartupInfoId& id, const char* bin,
     KStartupInfoData data;
     int desktop = get_current_desktop( X11_startup_notify_display );
     data.setDesktop( desktop );
-    data.setBin( bin );
+    data.setBin(QFile::decodeName(bin));
     KStartupInfo::sendChangeX( X11_startup_notify_display, id, data );
     XFlush( X11_startup_notify_display );
 }
@@ -402,32 +404,31 @@ static void complete_startup_info( KStartupInfoId& id, pid_t pid )
 QByteArray execpath_avoid_loops( const QByteArray& exec, int envc, const char* envs, bool avoid_loops )
 {
      QStringList paths;
+     const QRegExp pathSepRegExp(QString::fromLatin1("[:\b]"));
      if( envc > 0 ) /* use the passed environment */
      {
          const char* path = get_env_var( "PATH=", envc, envs );
          if( path != NULL )
-             paths = QString(path).split( QRegExp( "[:\b]" ));
+             paths = QFile::decodeName(path).split(pathSepRegExp);
+     } else {
+         paths = QString::fromLocal8Bit(qgetenv("PATH")).split(pathSepRegExp, QString::KeepEmptyParts);
      }
-     else
-         paths = QString::fromLocal8Bit( qgetenv("PATH") ).split( QRegExp( "[:\b]" ), QString::KeepEmptyParts );
-     QByteArray execpath = QFile::encodeName(
-         s_instance->dirs()->findExe( exec, paths.join( QLatin1String( ":" ))));
-     if( avoid_loops && !execpath.isEmpty())
-     {
-         int pos = execpath.lastIndexOf( '/' );
-         QString bin_path = execpath.left( pos );
+     QString execpath =
+         s_instance->dirs()->findExe(QFile::decodeName(exec), paths.join(QLatin1String(":")));
+     if (avoid_loops && !execpath.isEmpty()) {
+         const int pos = execpath.lastIndexOf(QLatin1Char('/'));
+         const QString bin_path = execpath.left(pos);
          for( QStringList::Iterator it = paths.begin();
               it != paths.end();
-              ++it )
-             if( ( *it ) == bin_path || ( *it ) == bin_path + '/' )
-             {
+              ++it ) {
+             if( *it == bin_path || *it == bin_path + QLatin1Char('/')) {
                  paths.erase( it );
                  break; // -->
              }
-         execpath = QFile::encodeName(
-             s_instance->dirs()->findExe( exec, paths.join( QString( ":" ))));
+         }
+         execpath = s_instance->dirs()->findExe(QFile::decodeName(exec), paths.join(QLatin1String(":")));
      }
-     return execpath;
+     return QFile::encodeName(execpath);
 }
 
 static pid_t launch(int argc, const char *_name, const char *args,
@@ -436,8 +437,8 @@ static pid_t launch(int argc, const char *_name, const char *args,
                     const char *tty=0, bool avoid_loops = false,
                     const char* startup_id_str = "0" )
 {
-  int launcher = 0;
-  QByteArray lib;
+  int starting_klauncher = 0;
+  QString lib;
   QByteArray name;
   QByteArray exec;
 
@@ -450,34 +451,31 @@ static pid_t launch(int argc, const char *_name, const char *args,
         perror("kdeinit4: socketpair() failed!\n");
         exit(255);
      }
-     launcher = 1;
+     starting_klauncher = 1;
   }
 
-  QByteArray libpath;
-  QByteArray execpath;
-  if (_name[0] != '/')
-  {
-     lib = name = _name;
-     exec = name;
-     libpath = QFile::encodeName(KLibLoader::findLibrary(lib.constData(), *s_instance));
-     execpath = execpath_avoid_loops( exec, envc, envs, avoid_loops );
-  }
-  else
-  {
-     lib = _name;
-     name = _name;
-     name = name.mid( name.lastIndexOf('/') + 1);
-     exec = _name;
-     if (lib.endsWith(".la"))
-        libpath = lib;
-     else
-        execpath = exec;
-  }
-  fprintf(stderr,"kdeinit4: preparing to launch %s\n", execpath.constData());
-  if (!args)
-  {
-    argc = 1;
-  }
+    QString libpath;
+    QByteArray execpath;
+    if (_name[0] != '/') {
+        name = _name;
+        lib = QFile::decodeName(name);
+        exec = name;
+        libpath = KLibLoader::findLibrary(lib, *s_instance);
+        execpath = execpath_avoid_loops(exec, envc, envs, avoid_loops);
+    } else {
+        name = _name;
+        lib = QFile::decodeName(name);
+        name = name.mid(name.lastIndexOf('/') + 1);
+        exec = _name;
+        if (lib.endsWith(QLatin1String(".la")))
+            libpath = lib;
+        else
+            execpath = exec;
+    }
+    fprintf(stderr,"kdeinit4: preparing to launch %s\n", execpath.constData());
+    if (!args) {
+        argc = 1;
+    }
 
   if (0 > pipe(d.fd))
   {
@@ -513,7 +511,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
      /** Child **/
      close(d.fd[0]);
      close_fds();
-     if (launcher)
+     if (starting_klauncher)
      {
         if (d.fd[1] == LAUNCHER_FD)
         {
@@ -530,17 +528,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
      if (cwd && *cwd)
         chdir(cwd);
      else {
-         // Can't use KGlobalSettings::documentPath() here, we don't have a main component set,
-         // and we don't want to set one; e.g. kioslaves will do that.
-         // So we have to duplicate this stuff from KGlobalSettings.
-         KConfigGroup g(s_instance->config(), "Paths");
-         const QString documentPath = g.readPathEntry("Documents",
-#ifdef Q_WS_WIN
-                                                       getWin32ShellFoldersPath("Personal")
-#else
-                                                       QDir::homePath());
-#endif
-         const QByteArray docPath = QFile::encodeName(documentPath);
+         const QByteArray docPath = QFile::encodeName(KGlobalSettings::documentPath());
          chdir(docPath.constData());
      }
 
@@ -596,6 +584,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
        }
        d.argv[argc] = 0;
 
+#ifndef SKIP_PROCTITLE
        /** Give the process a new name **/
 #ifdef Q_OS_LINUX
        /* set the process name, so that killall works like intended */
@@ -606,6 +595,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
            proctitle_set( "kdeinit4: %s%s", name.data(), procTitle.data() ? procTitle.data() : "" );
 #else
        proctitle_set( "kdeinit4: %s%s", name.data(), procTitle.data() ? procTitle.data() : "" );
+#endif
 #endif
      }
 
@@ -616,7 +606,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
      }
 
 
-     if ( !qgetenv("KDE_IS_PRELINKED").isEmpty() && !execpath.isEmpty() && !launcher)
+     if ( !qgetenv("KDE_IS_PRELINKED").isEmpty() && !execpath.isEmpty() && !starting_klauncher)
          libpath.truncate(0);
 
      QLibrary l(libpath);
@@ -629,13 +619,13 @@ static pid_t launch(int argc, const char *_name, const char *args,
           if (execpath.isEmpty())
           {
              // Error
-             QString errorMsg = i18n("Could not open library '%1'.\n%2", QFile::decodeName(libpath), ltdlError);
+             QString errorMsg = i18n("Could not open library '%1'.\n%2", libpath, ltdlError);
              exitWithErrorMsg(errorMsg);
           }
           else
           {
              // Print warning
-             fprintf(stderr, "Could not open library %s: %s\n", lib.data(),
+             fprintf(stderr, "Could not open library %s: %s\n", qPrintable(lib),
                      qPrintable(ltdlError) );
           }
        }
@@ -651,9 +641,9 @@ static pid_t launch(int argc, const char *_name, const char *args,
 
         setup_tty( tty );
 
-        QByteArray executable = execpath.data();
+        QByteArray executable = execpath;
 #ifdef Q_WS_MAC
-        QString bundlepath = s_instance->dirs()->findExe( execpath.data() );
+        QString bundlepath = s_instance->dirs()->findExe(QFile::decodeName(executable));
         if (!bundlepath.isEmpty())
            executable = QFile::encodeName(bundlepath);
 #endif
@@ -676,7 +666,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
             QString ltdlError = l.errorString();
             fprintf(stderr, "Could not find kdemain: %s\n", qPrintable(ltdlError) );
               QString errorMsg = i18n("Could not find 'kdemain' in '%1'.\n%2",
-                    QLatin1String(libpath), ltdlError);
+                                      libpath, ltdlError);
               exitWithErrorMsg(errorMsg);
            }
         }
@@ -706,7 +696,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
   default:
      /** Parent **/
      close(d.fd[1]);
-     if (launcher)
+     if (starting_klauncher)
      {
         close(d.launcher[1]);
         d.launcher_pid = d.fork;
@@ -768,9 +758,9 @@ static pid_t launch(int argc, const char *_name, const char *args,
        break;
      }
      close(d.fd[0]);
-     if (launcher && (d.result == 0))
+     if (starting_klauncher && (d.result == 0))
      {
-        // Trader launched successful
+        // klauncher launched successfully
         d.launcher_pid = d.fork;
      }
   }
@@ -1171,6 +1161,7 @@ static void handle_launcher_request(int sock = -1)
        }
    }
 
+   //kDebug() << "Got cmd" << request_header.cmd << commandToString(request_header.cmd);
    if (request_header.cmd == LAUNCHER_OK)
    {
       d.launcher_ok = true;
@@ -1399,7 +1390,8 @@ static void handle_requests(pid_t waitForPid)
       char c;
 
       /* Flush the pipe of death */
-      while( read(d.deadpipe[0], &c, 1) == 1);
+      while( read(d.deadpipe[0], &c, 1) == 1)
+        {}
 
       /* Handle dying children */
       do {
@@ -1495,18 +1487,18 @@ static void handle_requests(pid_t waitForPid)
 
 static void kdeinit_library_path()
 {
-   QStringList ltdl_library_path =
-     QFile::decodeName(qgetenv("LTDL_LIBRARY_PATH")).split(':',QString::SkipEmptyParts);
+   const QStringList ltdl_library_path =
+     QFile::decodeName(qgetenv("LTDL_LIBRARY_PATH")).split(QLatin1Char(':'),QString::SkipEmptyParts);
 #ifdef Q_OS_DARWIN
-   QStringList ld_library_path =
-     QFile::decodeName(qgetenv("DYLD_LIBRARY_PATH")).split(':',QString::SkipEmptyParts);
+   const QByteArray ldlibpath = qgetenv("DYLD_LIBRARY_PATH");
 #else
-   QStringList ld_library_path =
-     QFile::decodeName(qgetenv("LD_LIBRARY_PATH")).split(':',QString::SkipEmptyParts);
+   const QByteArray ldlibpath = qgetenv("LD_LIBRARY_PATH");
 #endif
+   const QStringList ld_library_path =
+     QFile::decodeName(ldlibpath).split(QLatin1Char(':'),QString::SkipEmptyParts);
 
    QByteArray extra_path;
-   QStringList candidates = s_instance->dirs()->resourceDirs("lib");
+   const QStringList candidates = s_instance->dirs()->resourceDirs("lib");
    for (QStringList::ConstIterator it = candidates.begin();
         it != candidates.end();
         ++it)
@@ -1516,7 +1508,7 @@ static void kdeinit_library_path()
           continue;
       if (ld_library_path.contains(d))
           continue;
-      if (d[d.length()-1] == '/')
+      if (d[d.length()-1] == QLatin1Char('/'))
       {
          d.truncate(d.length()-1);
          if (ltdl_library_path.contains(d))
@@ -1524,7 +1516,7 @@ static void kdeinit_library_path()
          if (ld_library_path.contains(d))
             continue;
       }
-      if ((d == "/lib") || (d == "/usr/lib"))
+      if ((d == QLatin1String("/lib")) || (d == QLatin1String("/usr/lib")))
          continue;
 
       QByteArray dir = QFile::encodeName(d);
@@ -1568,8 +1560,12 @@ static void kdeinit_library_path()
 #endif
 
    display.replace(':','_');
+#ifdef __APPLE__
+   display.replace('/','_');
+#endif
    // WARNING, if you change the socket name, adjust kwrapper too
-   QByteArray socketName = QFile::encodeName(KStandardDirs::locateLocal("socket", QString("kdeinit4_%1").arg(QLatin1String(display)), *s_instance));
+   const QString socketFileName = QString::fromLatin1("kdeinit4_%1").arg(QLatin1String(display));
+   QByteArray socketName = QFile::encodeName(KStandardDirs::locateLocal("socket", socketFileName, *s_instance));
    if (socketName.length() >= MAX_SOCK_FILE)
    {
      fprintf(stderr, "kdeinit4: Aborting. Socket name will be too long:\n");
@@ -1639,11 +1635,14 @@ int kdeinit_x_errhandler( Display *dpy, XErrorEvent *err )
     char errstr[256];
     // kdeinit almost doesn't use X, and therefore there shouldn't be any X error
     XGetErrorText( dpy, err->error_code, errstr, 256 );
-    fprintf(stderr, "kdeinit4: KDE detected X Error: %s %d\n"
+    fprintf(stderr, "kdeinit4(%d) : KDE detected X Error: %s %d\n"
                     "         Major opcode: %d\n"
                     "         Minor opcode: %d\n"
                     "         Resource id:  0x%lx\n",
-            errstr, err->error_code, err->request_code, err->minor_code, err->resourceid );
+            getpid(), errstr, err->error_code, err->request_code, err->minor_code, err->resourceid );
+
+    //kDebug() << kBacktrace();
+
 #else
     Q_UNUSED(dpy);
     Q_UNUSED(err);
@@ -1724,7 +1723,9 @@ static void secondary_child_handler(int)
 
 int main(int argc, char **argv, char **envp)
 {
-   int i;
+    setlocale (LC_ALL, "");
+    setlocale (LC_NUMERIC, "C");
+
    pid_t pid;
    bool do_fork = true;
    int launch_klauncher = 1;
@@ -1734,7 +1735,7 @@ int main(int argc, char **argv, char **envp)
 
    /** Save arguments first... **/
    char **safe_argv = (char **) malloc( sizeof(char *) * argc);
-   for(i = 0; i < argc; i++)
+   for(int i = 0; i < argc; i++)
    {
       safe_argv[i] = strcpy((char*)malloc(strlen(argv[i])+1), argv[i]);
       if (strcmp(safe_argv[i], "--no-klauncher") == 0)
@@ -1752,6 +1753,12 @@ int main(int argc, char **argv, char **envp)
          d.suicide = true;
       if (strcmp(safe_argv[i], "--exit") == 0)
          keep_running = 0;
+      if (strcmp(safe_argv[i], "--version") == 0)
+      {
+	 printf("Qt: %s\n", qVersion());
+	 printf("KDE: %s\n", KDE_VERSION_STRING);
+	 exit(0);
+      }
       if (strcmp(safe_argv[i], "--help") == 0)
       {
         printf("Usage: kdeinit4 [options]\n");
@@ -1764,6 +1771,7 @@ int main(int argc, char **argv, char **envp)
      // printf("    --no-klauncher    Do not start klauncher\n");
         printf("    --no-kded         Do not start kded\n");
         printf("    --suicide         Terminate when no KDE applications are left running\n");
+	printf("    --version         Show version information\n");
      // printf("    --exit            Terminate when kded has run\n");
         exit(0);
       }
@@ -1814,7 +1822,10 @@ int main(int argc, char **argv, char **envp)
    s_instance = new KComponentData("kdeinit4", QByteArray(), KComponentData::SkipMainComponentRegistration);
 
    /** Prepare to change process name **/
+#ifndef SKIP_PROCTITLE
    proctitle_init(argc, argv, envp);
+#endif
+
    kdeinit_library_path();
    // Don't make our instance the global instance
    // (do it only after kdeinit_library_path, that one indirectly uses KConfig,
@@ -1850,7 +1861,7 @@ int main(int argc, char **argv, char **envp)
 #ifdef Q_WS_X11
    if (!d.suicide && qgetenv("KDE_IS_PRELINKED").isEmpty())
    {
-       QString konq = KStandardDirs::locate("lib", "libkonq.so.5", *s_instance);
+       QString konq = KStandardDirs::locate("lib", QLatin1String("libkonq.so.5"), *s_instance);
        // can't use KLibLoader here as it would unload the library
        // again
        if (!konq.isEmpty()) {
@@ -1875,8 +1886,6 @@ int main(int argc, char **argv, char **envp)
 
    {
       QFont::initialize();
-      setlocale (LC_ALL, "");
-      setlocale (LC_NUMERIC, "C");
 #ifdef Q_WS_X11
       if (XSupportsLocale ())
       {
@@ -1897,7 +1906,7 @@ int main(int argc, char **argv, char **envp)
       handle_requests(pid);
    }
 
-   for(i = 1; i < argc; i++)
+   for(int i = 1; i < argc; i++)
    {
       if (safe_argv[i][0] == '+')
       {
@@ -1921,13 +1930,15 @@ int main(int argc, char **argv, char **envp)
    }
 
    /** Free arguments **/
-   for(i = 0; i < argc; i++)
+   for(int i = 0; i < argc; i++)
    {
       free(safe_argv[i]);
    }
    free (safe_argv);
 
+#ifndef SKIP_PROCTITLE
    proctitle_set("kdeinit4 Running...");
+#endif
 
    if (!keep_running)
       return 0;
