@@ -4,7 +4,7 @@
    Copyright (C) 2000,2001 George Staikos <staikos@kde.org>
    Copyright (C) 2001,2002 Hamish Rodda <rodda@kde.org>
    Copyright (C) 2007      Daniel Nicoletti <mirttex@users.sourceforge.net>
-
+   Copyright (C) 2008,2009 Andreas Hartmetz <ahartmetz@gmail.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -26,6 +26,7 @@
 #define HTTP_H_
 
 
+#include <sys/types.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <string.h>
@@ -41,6 +42,11 @@
 #include "kio/tcpslavebase.h"
 #include "kio/http.h"
 
+// HeaderTokenizer declarations
+#include "parsinghelpers.h"
+// KHttpAuthentication & KHttpAuthenticationOutcome declarations
+#include "httpauthentication.h"
+
 class QDomNodeList;
 
 namespace KIO {
@@ -59,30 +65,7 @@ public:
   enum HTTP_REV    {HTTP_None, HTTP_Unknown, HTTP_10, HTTP_11, SHOUTCAST};
 
   /** Authorization method used **/
-  enum HTTP_AUTH   {AUTH_None, AUTH_Basic, AUTH_NTLM, AUTH_Digest, AUTH_Negotiate};
-
-  /** HTTP / DAV method **/
-  // Removed to interfaces/kio/http.h
-  //enum HTTP_METHOD {HTTP_GET, HTTP_PUT, HTTP_POST, HTTP_HEAD, HTTP_DELETE,
-  //                  HTTP_OPTIONS, DAV_PROPFIND, DAV_PROPPATCH, DAV_MKCOL,
-  //                  DAV_COPY, DAV_MOVE, DAV_LOCK, DAV_UNLOCK, DAV_SEARCH };
-
-  /** State of the current Connection **/
-  struct HTTPState
-  {
-    HTTPState ()
-    {
-      port = 0;
-      doProxy = false;
-    }
-
-    QString hostname;
-    QString encoded_hostname;
-    short unsigned int port;
-    QString user;
-    QString passwd;
-    bool  doProxy;
-  };
+  enum AUTH_SCHEME   {AUTH_None, AUTH_Basic, AUTH_NTLM, AUTH_Digest, AUTH_Negotiate};
 
   /** DAV-specific request elements for the current connection **/
   struct DAVRequest
@@ -98,95 +81,122 @@ public:
     int depth;
   };
 
+  struct CacheTag
+  {
+    CacheTag()
+    {
+      useCache = false;
+      readFromCache = false;
+      writeToCache = false;
+      isExpired = false;
+      bytesCached = 0;
+      gzs = 0;
+      expireDateOffset = 0;
+      expireDate = 0;
+      creationDate = 0;
+    }
+
+    KIO::CacheControl policy;    // ### initialize in the constructor?
+    bool useCache; // Whether the cache is active
+    bool readFromCache; // Whether the request is to be read from file.
+    bool writeToCache; // Whether the request is to be written to file.
+    bool isExpired;
+    long bytesCached;
+    QString etag; // entity tag header as described in the HTTP standard.
+    QString file; // cache entry file belonging to this URL.
+    gzFile gzs; // gzip stream of the cache entry
+    QString lastModified; // Last modified.
+    long expireDateOffset; // Position in the cache entry where the
+                           // 16 byte expire date is stored.
+    time_t expireDate; // Date when the cache entry will expire
+    time_t creationDate; // Date when the cache entry was created
+    QString charset;
+  };
+
   /** The request for the current connection **/
   struct HTTPRequest
   {
     HTTPRequest ()
     {
-      port = 0;
       method = KIO::HTTP_UNKNOWN;
       offset = 0;
       endoffset = 0;
-      doProxy = false;
-      allowCompressedPage = false;
-      disablePassDlg = false;
-      bNoAuth = false;
-      bUseCache = false;
-      bCachedRead = false;
-      bCachedWrite = false;
-      fcache = 0;
-      bMustRevalidate = false;
-      cacheExpireDateOffset = 0;
-      bErrorPage = false;
-      bUseCookiejar = false;
-      expireDate = 0;
-      creationDate = 0;
-      bytesCached=0;
+      allowTransferCompression = false;
+      disablePassDialog = false;
+      doNotAuthenticate = false;
+      preferErrorPage = false;
+      useCookieJar = false;
     }
 
-    QString hostname;
-    QString encoded_hostname;
-    short unsigned int port;
-    QString user;
-    QString passwd;
-    QString path;
-    QString query;
+    KUrl url;
+    QString encoded_hostname; //### can be calculated on-the-fly
+    // Persistent connections
+    bool isKeepAlive;
+    int keepAliveTimeout;   // Timeout in seconds.
+
     KIO::HTTP_METHOD method;
-    KIO::CacheControl cache;
     KIO::filesize_t offset;
     KIO::filesize_t endoffset;
-    KUrl url;
-    QString window;                 // Window Id this request is related to.
+    QString windowId;                 // Window Id this request is related to.
+    // Header fields
     QString referrer;
     QString charsets;
     QString languages;
     QString userAgent;
+    // Previous and current response codes
+    unsigned int responseCode;
+    unsigned int prevResponseCode;
+    // Miscellaneous
     QString id;
     DAVRequest davData;
-    bool doProxy;
-    bool allowCompressedPage;
-    bool disablePassDlg;
-    bool bNoAuth; // Do not authenticate
-
-    // Indicates whether an error-page or error-msg should is preferred.
-    bool bErrorPage;
-
-    // Cookie flags
-    bool bUseCookiejar;
-
-    // Cache related
-    bool bUseCache; // Whether the cache is active
-    bool bCachedRead; // Whether the file is to be read from m_fcache.
-    bool bCachedWrite; // Whether the file is to be written to m_fcache.
-    bool bMustRevalidate; // Cache entry is expired.
-    QString cef; // Cache Entry File belonging to this URL.
-    gzFile fcache; // File stream of a cache entry
-    QString etag; // ETag header.
-    QString lastModified; // Last modified.
-    long cacheExpireDateOffset; // Position in the cache entry where the
-                                  // 16 byte expire date is stored.
-    long bytesCached;
-    time_t expireDate; // Date when the cache entry will expire
-    time_t creationDate; // Date when the cache entry was created
-    QString strCharset;
+    KUrl proxyUrl;
+    bool isPersistentProxyConnection;
+    bool allowTransferCompression;
+    bool disablePassDialog;
+    bool doNotAuthenticate;
+    // Indicates whether an error page or error message is preferred.
+    bool preferErrorPage;
 
+    // Use the cookie jar (or pass cookies to the application as metadata instead)
+    bool useCookieJar;
     // Cookie flags
     enum { CookiesAuto, CookiesManual, CookiesNone } cookieMode;
+
+    CacheTag cacheTag;
   };
 
-  struct DigestAuthInfo
+  /** State of the current connection to the server **/
+  struct HTTPServerState
   {
-    QByteArray nc;
-    QByteArray qop;
-    QByteArray realm;
-    QByteArray nonce;
-    QByteArray method;
-    QByteArray cnonce;
-    QByteArray username;
-    QByteArray password;
-    KUrl::List digestURI;
-    QByteArray algorithm;
-    QByteArray entityBody;
+    HTTPServerState()
+    {
+      isKeepAlive = false;
+      isPersistentProxyConnection = false;
+    }
+
+    void initFrom(const HTTPRequest &request)
+    {
+      url = request.url;
+      encoded_hostname = request.encoded_hostname;
+      isKeepAlive = request.isKeepAlive;
+      proxyUrl = request.proxyUrl;
+      isPersistentProxyConnection = request.isPersistentProxyConnection;
+    }
+
+    void clear()
+    {
+      url.clear();
+      encoded_hostname.clear();
+      proxyUrl.clear();
+      isKeepAlive = false;
+      isPersistentProxyConnection = false;
+    }
+
+    KUrl url;
+    QString encoded_hostname;
+    KUrl proxyUrl;
+    bool isKeepAlive;
+    bool isPersistentProxyConnection;
   };
 
 //---------------------- Re-implemented methods ----------------
@@ -245,7 +255,7 @@ public:
 
   void post( const KUrl& url );
   void multiGet(const QByteArray &data);
-  bool checkRequestUrl( const KUrl& );
+  bool maybeSetRequestUrl(const KUrl &);
   void cacheUpdate( const KUrl &url, bool nocache, time_t expireDate);
 
   void httpError(); // Generate error message based on response code
@@ -254,7 +264,9 @@ public:
 
 protected Q_SLOTS:
   void slotData(const QByteArray &);
-  void error( int _errid, const QString &_text );
+  void error(int errid, const QString &text);
+  void proxyAuthenticationForSocket(const QNetworkProxy &, QAuthenticator *);
+  void saveProxyAuthenticationForSocket();
 
 protected:
   int readChunked();    // Read a chunk
@@ -262,25 +274,11 @@ protected:
   int readUnlimited();  // Read as much as possible.
 
   /**
-    * A "smart" wrapper around write that will use SSL_write or
-    * write(2) depending on whether you've got an SSL connection or not.
-    * The only shortcomming is that it uses the "global" file handles and
-    * soforth.  So you can't really use this on individual files/sockets.
+    * A thin wrapper around TCPSlaveBase::write() that will retry writing as
+    * long as no error occurs.
     */
   ssize_t write(const void *buf, size_t nbytes);
 
-  /**
-    * Another "smart" wrapper, this time around read that will
-    * use SSL_read or read(2) depending on whether you've got an
-    * SSL connection or not.
-    */
-  ssize_t read (void *b, size_t nbytes);
-
-  char *gets (char *str, int size);
-
-  void setRewindMarker();
-  void rewind();
-
   /**
     * Add an encoding on to the appropriate stack this
     * is nececesary because transfer encodings and
@@ -288,18 +286,28 @@ protected:
     */
   void addEncoding(const QString &, QStringList &);
 
-  void configAuth( char *, bool );
+  //void processAuthenticationRequest();
 
-  bool httpOpen();             // Open transfer
+  // The methods between here and sendQuery() are helpers for sendQuery().
+
+  // Return true if the request is already "done", false otherwise.
+  // *sucesss will be set to true if the page was found, false otherwise.
+  bool satisfyRequestFromCache(bool *success);
+  QString formatRequestUri() const;
+  // create HTTP authentications response(s), if any
+  QString authenticationHeader();
+  bool sendQuery();
+  
   void httpClose(bool keepAlive);  // Close transfer
-
   bool httpOpenConnection();   // Open connection
   void httpCloseConnection();  // Close connection
-  void httpCheckConnection();  // Check whether to keep connection.
+  bool httpShouldCloseConnection();  // Check whether to keep or close the connection.
 
   void forwardHttpResponseHeader();
 
-  bool readHeader();
+  // Helper for readResponseHeader - fix common mimetype errors by webservers.
+  void fixupResponseMimetype();
+  bool readResponseHeader();
   bool readHeaderFromCache();
   void parseContentDisposition(const QString &disposition);
 
@@ -345,7 +353,7 @@ protected:
   QString findCookies( const QString &url);
 
   /**
-   * Do a cache lookup for the current url. (m_state.url)
+   * Do a cache lookup for the current url. (m_server.url)
    *
    * @param readWrite If true, file is opened read/write.
    *                  If false, file is opened read-only.
@@ -358,7 +366,7 @@ protected:
   gzFile checkCacheEntry(bool readWrite = false);
 
   /**
-   * Create a cache entry for the current url. (m_state.url)
+   * Create a cache entry for the current url. (m_server.url)
    *
    * Set the contents type of the cache entry to 'mimetype'.
    */
@@ -387,16 +395,18 @@ protected:
   void cleanCache();
 
   /**
-   * Performs a GET HTTP request.
+   * Do everything proceedUntilResponseHeader does, and also get the response body.
+   * This is being used as a replacement for proceedUntilResponseHeader() in
+   * situations where we actually expect the response to have a body / payload data.
    */
   // where dataInternal == true, the content is to be made available
   // to an internal function.
-  void retrieveContent( bool dataInternal = false );
+  void proceedUntilResponseContent( bool dataInternal = false );
 
   /**
-   * Performs a HEAD HTTP request.
+   * Ensure we are connected, send our query, and get the response header.
    */
-  bool retrieveHeader(bool close_connection = true);
+  bool proceedUntilResponseHeader();
 
   /**
    * Resets any per session settings.
@@ -404,49 +414,18 @@ protected:
   void resetSessionSettings();
 
   /**
-   * Resets settings related to parsing a response.
+   * Resets variables related to parsing a response.
    */
-  void resetResponseSettings();
+  void resetResponseParsing();
 
   /**
-   * Resets any per connection settings.  These are different from
-   * per-session settings in that they must be invalidates every time
+   * Resets any per connection settings. These are different from
+   * per-session settings in that they must be invalidated every time
    * a request is made, e.g. a retry to re-send the header to the
    * server, as compared to only when a new request arrives.
    */
   void resetConnectionSettings();
 
-  /**
-   * Returns any pre-cached proxy authentication info
-   * info in HTTP header format.
-   */
-  QString proxyAuthenticationHeader();
-
-  /**
-   * Retrieves authorization info from cache or user.
-   */
-  bool getAuthorization();
-
-  /**
-   * Saves valid authorization info in the cache daemon.
-   */
-  void saveAuthorization();
-
-  /**
-   * Creates the entity-header for Basic authentication.
-   */
-  QString createBasicAuth( bool isForProxy = false );
-
-  /**
-   * Creates the entity-header for Digest authentication.
-   */
-  QString createDigestAuth( bool isForProxy = false );
-
-  /**
-   * Creates the entity-header for NTLM authentication.
-   */
-  QString createNTLMAuth( bool isForProxy = false );
-
   /**
    * Creates the entity-header for Negotiate authentication.
    */
@@ -457,65 +436,47 @@ protected:
    */
   QByteArray gssError( int major_status, int minor_status );
 
-  /**
-   * Calcualtes the message digest response based on RFC 2617.
-   */
-  void calculateResponse( DigestAuthInfo &info, QByteArray &Response );
-
-  /**
-   * Prompts the user for authorization retry.
-   */
-  bool retryPrompt();
-
   /**
    * Creates authorization prompt info.
    */
-  void promptInfo( KIO::AuthInfo& info );
+  void fillPromptInfo(KIO::AuthInfo *info);
 
 protected:
-  HTTPState m_state;
+  HTTPServerState m_server;
   HTTPRequest m_request;
-  QList<HTTPRequest*> m_requestQueue;
+  QList<HTTPRequest> m_requestQueue;
   quint16 m_defaultPort;
 
   // Processing related
   KIO::filesize_t m_iSize; // Expected size of message
   KIO::filesize_t m_iBytesLeft; // # of bytes left to receive in this message.
   KIO::filesize_t m_iContentLeft; // # of content bytes left
-  QByteArray m_bufReceive; // Receive buffer
-  char m_lineBuf[1024];
-  char m_rewindBuf[8192];
-  size_t m_rewindCount;
-  size_t m_lineCount;
-  size_t m_lineCountUnget;
-  char *m_linePtr;
-  char *m_lineBufUnget;
-  char *m_linePtrUnget;
+  QByteArray m_receiveBuf; // Receive buffer
   bool m_dataInternal; // Data is for internal consumption
-  bool m_bChunked; // Chunked transfer encoding
+  bool m_isChunked; // Chunked transfer encoding
 
-  bool m_bBusy; // Busy handling request queue.
-  bool m_bEOF;
-  bool m_bEOD;
+  bool m_isBusy; // Busy handling request queue.
+  bool m_isEOF;
+  bool m_isEOD;
 
   // First request on a connection
-  bool m_bFirstRequest;
+  bool m_isFirstRequest;
 
 //--- Settings related to a single response only
-  bool m_bRedirect; // Indicates current request is a redirection
+  bool m_isRedirection; // Indicates current request is a redirection
   QStringList m_responseHeaders; // All headers
 
 
   // Language/Encoding related
-  QStringList m_qTransferEncodings;
-  QStringList m_qContentEncodings;
-  QString m_sContentMD5;
-  QString m_strMimeType;
+  QStringList m_transferEncodings;
+  QStringList m_contentEncodings;
+  QString m_contentMD5;
+  QString m_mimeType;
 
 
 //--- WebDAV
   // Data structure to hold data which will be passed to an internal func.
-  QByteArray m_bufWebDavData;
+  QByteArray m_webDavDataBuf;
   QStringList m_davCapabilities;
 
   bool m_davHostOk;
@@ -528,9 +489,9 @@ protected:
 
 
   // Holds the POST data so it won't get lost on if we
-  // happend to get a 401/407 response when submitting,
+  // happend to get a 401/407 response when submitting
   // a form.
-  QByteArray m_bufPOST;
+  QByteArray m_POSTbuf;
 
   // Cache related
   int m_maxCacheAge; // Maximum age of a cache entry.
@@ -538,45 +499,39 @@ protected:
   QString m_strCacheDir; // Location of the cache.
 
 
-
 //--- Proxy related members
-  bool m_bUseProxy;
-  bool m_bNeedTunnel; // Whether we need to make a SSL tunnel
-  bool m_bIsTunneled; // Whether we have an active SSL tunnel
-  bool m_bProxyAuthValid;
-  int m_iProxyPort;
-  KUrl m_proxyURL;
-  QString m_strProxyRealm;
 
   // Operation mode
   QByteArray m_protocol;
 
-  // Authentication
-  QString m_strRealm;
-  QString m_strAuthorization;
-  QString m_strProxyAuthorization;
-  HTTP_AUTH Authentication;
-  HTTP_AUTH ProxyAuthentication;
-  short unsigned int m_iProxyAuthCount;
-  short unsigned int m_iWWWAuthCount;
-  bool m_bUnauthorized;
-
-  // Persistent proxy connections
-  bool m_bPersistentProxyConnection;
+  //TODO use PrevRequest fully
+  //TODO isAuthValid/m_isUnauthorized -> auth.scheme != AUTH_None ?
+#if 0
+  struct AuthState
+  {
+    AUTH_SCHEME scheme;
+    QString realm;
+    QString authorization;
+    int authCount;
+  };
+#endif
 
+  KAbstractHttpAuthentication *m_wwwAuth;
+  KAbstractHttpAuthentication *m_proxyAuth;
+  // For proxy auth when it's handled by the Qt/KDE socket classes
+  QAuthenticator *m_socketProxyAuth;
 
   // Indicates whether there was some connection error.
-  bool m_bError;
-
-  // Persistent connections
-  bool m_bKeepAlive;
-  int m_keepAliveTimeout; // Timeout in seconds.
-
-  // Previous and current response codes
-  unsigned int m_responseCode;
-  unsigned int m_prevResponseCode;
+  bool m_isError;
 
   // Values that determine the remote connection timeouts.
   int m_remoteRespTimeout;
+
+
+  QByteArray m_unreadBuf;
+  void clearUnreadBuffer();
+  void unread(char *buf, size_t size);
+  size_t readBuffered(char *buf, size_t size);
+  bool readDelimitedText(char *buf, int *idx, int end, int numNewlines);
 };
 #endif
